@@ -934,15 +934,11 @@ galago_account_get_contact(const GalagoAccount *account, const char *username,
 	if (contact == NULL && query && GALAGO_OBJECT_IS_REMOTE(account) &&
 		!galago_is_daemon() && galago_is_connected())
 	{
-		galago_context_push(galago_object_get_context(GALAGO_OBJECT(account)));
-
 		contact = galago_dbus_send_message_with_reply(GALAGO_OBJECT(account),
 			"GetContact",
 			galago_value_new_object(GALAGO_TYPE_ACCOUNT, NULL),
 			galago_value_new(GALAGO_VALUE_TYPE_STRING, &username, NULL),
 			NULL);
-
-		galago_context_pop();
 	}
 
 	return contact;
@@ -974,16 +970,12 @@ galago_account_get_contacts(const GalagoAccount *account, gboolean query)
 	{
 		GList *temp;
 
-		galago_context_push(galago_object_get_context(GALAGO_OBJECT(account)));
-
 		temp = galago_dbus_send_message_with_reply(GALAGO_OBJECT(account),
 			"GetContacts",
 			galago_value_new_list(GALAGO_TYPE_OBJECT, NULL,
 								  (void *)GALAGO_TYPE_ACCOUNT),
 			NULL);
 		g_list_free(temp);
-
-		galago_context_pop();
 	}
 
 	return account->priv->contacts;
@@ -1023,22 +1015,95 @@ galago_account_create_presence(GalagoAccount *account)
 GalagoPresence *
 galago_account_get_presence(const GalagoAccount *account, gboolean query)
 {
+	GalagoPresence *presence;
+
 	g_return_val_if_fail(account != NULL,            NULL);
 	g_return_val_if_fail(GALAGO_IS_ACCOUNT(account), NULL);
 
-	if (account->priv->presence == NULL && query &&
+	presence = account->priv->presence;
+
+	if (presence == NULL && query &&
 		GALAGO_OBJECT_IS_REMOTE(account) && !galago_is_daemon() &&
 		galago_is_connected())
 	{
-		account->priv->presence =
+		presence =
 			galago_dbus_send_message_with_reply(GALAGO_OBJECT(account),
 				"GetPresence",
 				galago_value_new_object(GALAGO_TYPE_PRESENCE, NULL),
 				NULL);
-		g_object_notify(G_OBJECT(account), "presence");
 	}
 
-	return account->priv->presence;
+	return presence;
+}
+
+/**
+ * galago_account_get_presence_async
+ * @account:   The account.
+ * @cb:        The callback function that will be passed the presence.
+ * @user_data: Custom data to pass to the callback function.
+ * @free_func: Optional function to free @user_data when the request completes.
+ *
+ * Asynchronously retrieves the presence associated with the specified
+ * account.
+ *
+ * For example:
+ * <informalexample>
+ * <programlisting>
+ *  static void
+ *  presence_received_cb(GalagoAccount *account,
+ *                       GalagoPresence *presence,
+ *                       gpointer user_data)
+ *  {
+ *  }
+ *  
+ *  static void
+ *  get_presence(GalagoAccount *account)
+ *  {
+ *      GalagoCallHandle handle;
+ *  
+ *      handle = galago_account_get_presence_async(account,
+ *                                                 presence_received_cb,
+ *                                                 NULL, NULL);
+ *  }
+ * </programlisting>
+ * </informalexample>
+ *
+ * Returns: A valid #GalagoCallHandle on success. On error, this will return
+ *          %GALAGO_CALL_HANDLE_INVALID.
+ *
+ * Since: 0.5.2
+ */
+GalagoCallHandle
+galago_account_get_presence_async(const GalagoAccount *account,
+								  GalagoAccountPresenceCb cb,
+								  gpointer user_data,
+								  GFreeFunc free_func)
+{
+	GalagoPresence *presence;
+	GalagoCallHandle handle = GALAGO_CALL_HANDLE_INVALID;
+
+	g_return_val_if_fail(account  != NULL,                      handle);
+	g_return_val_if_fail(GALAGO_IS_ACCOUNT(account),            handle);
+	g_return_val_if_fail(cb != NULL,                            handle);
+
+	presence = galago_account_get_presence(account, FALSE);
+
+	if (presence != NULL)
+	{
+		cb((GalagoAccount *)account, presence, user_data);
+		handle = galago_calls_request_dummy_handle();
+	}
+	else if (GALAGO_OBJECT_IS_REMOTE(account) && !galago_is_daemon() &&
+			 galago_is_connected())
+	{
+		handle = galago_dbus_send_message_with_reply_async(
+			GALAGO_OBJECT(account), "GetPresence", G_CALLBACK(cb),
+			user_data, free_func, g_cclosure_marshal_VOID__POINTER,
+			galago_value_new_object(GALAGO_TYPE_PRESENCE, NULL),
+			NULL);
+	}
+
+	return handle;
 }
 
 /**
@@ -1087,22 +1152,93 @@ galago_account_set_avatar(GalagoAccount *account, GalagoImage *avatar)
 GalagoImage *
 galago_account_get_avatar(const GalagoAccount *account, gboolean query)
 {
+	GalagoImage *avatar;
+
 	g_return_val_if_fail(account != NULL,            NULL);
 	g_return_val_if_fail(GALAGO_IS_ACCOUNT(account), NULL);
 
-	if (account->priv->avatar == NULL && query &&
-		GALAGO_OBJECT_IS_REMOTE(account) && !galago_is_daemon() &&
-		galago_is_connected())
+	avatar = account->priv->avatar;
+
+	if (avatar == NULL && query && GALAGO_OBJECT_IS_REMOTE(account) &&
+		!galago_is_daemon() && galago_is_connected())
 	{
-		account->priv->avatar =
-			galago_dbus_send_message_with_reply(GALAGO_OBJECT(account),
-				"GetAvatar",
-				galago_value_new_object(GALAGO_TYPE_IMAGE, NULL),
-				NULL);
-		g_object_notify(G_OBJECT(account), "avatar");
+		avatar = galago_dbus_send_message_with_reply(GALAGO_OBJECT(account),
+			"GetAvatar",
+			galago_value_new_object(GALAGO_TYPE_IMAGE, NULL),
+			NULL);
 	}
 
-	return account->priv->avatar;
+	return avatar;
+}
+
+/**
+ * galago_account_get_avatar_async
+ * @account:   The account.
+ * @cb:        The callback function that will be passed the avatar.
+ * @user_data: Custom data to pass to the callback function.
+ * @free_func: Optional function to free @user_data when the request completes.
+ *
+ * Asynchronously retrieves the avatar associated with the specified
+ * account.
+ *
+ * For example:
+ * <informalexample>
+ * <programlisting>
+ *  static void
+ *  avatar_received_cb(GalagoAccount *account,
+ *                     GalagoAvatar *avatar,
+ *                     gpointer user_data)
+ *  {
+ *  }
+ *  
+ *  static void
+ *  get_avatar(GalagoAccount *account)
+ *  {
+ *      GalagoCallHandle handle;
+ *  
+ *      handle = galago_account_get_avatar_async(account,
+ *                                               avatar_received_cb,
+ *                                               NULL, NULL);
+ *  }
+ * </programlisting>
+ * </informalexample>
+ *
+ * Returns: A valid #GalagoCallHandle on success. On error, this will return
+ *          %GALAGO_CALL_HANDLE_INVALID.
+ *
+ * Since: 0.5.2
+ */
+GalagoCallHandle
+galago_account_get_avatar_async(const GalagoAccount *account,
+								GalagoAccountAvatarCb cb,
+								gpointer user_data,
+								GFreeFunc free_func)
+{
+	GalagoImage *avatar;
+	GalagoCallHandle handle = GALAGO_CALL_HANDLE_INVALID;
+
+	g_return_val_if_fail(account  != NULL,                      handle);
+	g_return_val_if_fail(GALAGO_IS_ACCOUNT(account),            handle);
+	g_return_val_if_fail(cb != NULL,                            handle);
+
+	avatar = galago_account_get_avatar(account, FALSE);
+
+	if (avatar != NULL)
+	{
+		cb((GalagoAccount *)account, avatar, user_data);
+		handle = galago_calls_request_dummy_handle();
+	}
+	else if (GALAGO_OBJECT_IS_REMOTE(account) && !galago_is_daemon() &&
+			 galago_is_connected())
+	{
+		handle = galago_dbus_send_message_with_reply_async(
+			GALAGO_OBJECT(account), "GetAvatar", G_CALLBACK(cb),
+			user_data, free_func, g_cclosure_marshal_VOID__POINTER,
+			galago_value_new_object(GALAGO_TYPE_IMAGE, NULL),
+			NULL);
+	}
+
+	return handle;
 }
 
 void
