@@ -31,6 +31,11 @@ typedef struct _GalagoAccountPrivate GalagoAccountPrivate;
 #include <libgalago/galago-presence.h>
 #include <libgalago/galago-service.h>
 
+typedef void (*GalagoAccountPresenceCb)(GalagoAccount *, GalagoPresence *,
+										gpointer);
+typedef void (*GalagoAccountAvatarCb)(GalagoAccount *, GalagoImage *,
+									  gpointer);
+
 struct _GalagoAccount
 {
 	GalagoObject parent_object;
@@ -113,10 +118,20 @@ GList *galago_account_get_contacts(const GalagoAccount *account,
 GalagoPresence *galago_account_create_presence(GalagoAccount *account);
 GalagoPresence *galago_account_get_presence(const GalagoAccount *account,
 											gboolean query);
+GalagoCallHandle galago_account_get_presence_async(
+	const GalagoAccount *account,
+	GalagoAccountPresenceCb cb,
+	gpointer user_data,
+	GFreeFunc free_func);
 
 void galago_account_set_avatar(GalagoAccount *account, GalagoImage *avatar);
 GalagoImage *galago_account_get_avatar(const GalagoAccount *account,
 									   gboolean query);
+GalagoCallHandle galago_account_get_avatar_async(
+	const GalagoAccount *account,
+	GalagoAccountAvatarCb cb,
+	gpointer user_data,
+	GFreeFunc free_func);
 
 G_END_DECLS
 
