@@ -231,7 +231,7 @@ galago_core_class_init(GalagoCoreClass *klass)
 					 NULL, NULL,
 					 g_cclosure_marshal_VOID__OBJECT,
 					 G_TYPE_NONE, 1,
-					 GALAGO_TYPE_SERVICE);
+					 GALAGO_TYPE_PERSON);
 
 	/**
 	 * GalagoCore::person-removed:
@@ -248,7 +248,7 @@ galago_core_class_init(GalagoCoreClass *klass)
 					 NULL, NULL,
 					 g_cclosure_marshal_VOID__OBJECT,
 					 G_TYPE_NONE, 1,
-					 GALAGO_TYPE_SERVICE);
+					 GALAGO_TYPE_PERSON);
 
 	/**
 	 * GalagoCore::calc-priority-account:
@@ -783,8 +783,10 @@ _galago_core_connect(void)
 
 	dbus_bus_add_match(_core->priv->dbus_conn,
 					   "type='signal',"
+					   "sender='" DBUS_SERVICE_DBUS "',"
 					   "interface='" DBUS_INTERFACE_DBUS "',"
-					   "sender='" DBUS_SERVICE_DBUS "'",
+					   "member='NameOwnerChanged',"
+					   "arg0='" GALAGO_DBUS_SERVICE "'",
 					   &error);
 
 	if (dbus_error_is_set(&error))
@@ -968,7 +970,10 @@ galago_uninit(void)
 	_core->priv->init_ref_count--;
 
 	if (_core->priv->init_ref_count == 0)
+	{
 		galago_object_destroy(GALAGO_OBJECT(_core));
+		_core = NULL;
+	}
 }
 
 /**
@@ -1243,6 +1248,78 @@ galago_get_service(const char *id, GalagoOrigin origin, gboolean query)
 	return service;
 }
 
+/**
+ * galago_get_service_async:
+ * @id:        The service ID.
+ * @cb:        The callback function that will be passed the service.
+ * @user_data: Custom data to pass to the callback function.
+ * @free_func: Optional function to free @user_data when the request completes.
+ *
+ * Asynchronously retrieves the service with the specified ID.
+ *
+ * For example:
+ * <informalexample>
+ * <programlisting>
+ *  static void
+ *  service_received_cb(GalagoCore *core,
+ *                      GalagoService *service,
+ *                      gpointer user_data)
+ *  {
+ *  }
+ *  
+ *  static void
+ *  get_jabber_service(void)
+ *  {
+ *      GalagoCallHandle handle;
+ *  
+ *      handle = galago_get_service_async("jabber", service_received_cb,
+ *                                        NULL, NULL);
+ *  }
+ * </programlisting>
+ * </informalexample>
+ *
+ * Returns: A valid #GalagoCallHandle on success. On error, this will return
+ *          %GALAGO_CALL_HANDLE_INVALID.
+ *
+ * Since: 0.5.2
+ */
+GalagoCallHandle
+galago_get_service_async(const char *id,
+						 GalagoCoreServiceCb cb,
+						 gpointer user_data,
+						 GFreeFunc free_func)
+{
+	GalagoService *service;
+	GalagoCallHandle handle = GALAGO_CALL_HANDLE_INVALID;
+
+	g_return_val_if_fail(galago_is_initted(),       handle);
+	g_return_val_if_fail(id != NULL && *id != '\0', handle);
+	g_return_val_if_fail(cb != NULL,                handle);
+
+	galago_context_push(galago_object_get_context(GALAGO_OBJECT(_core)));
+
+	service = galago_context_get_service(id, GALAGO_REMOTE);
+
+	if (service != NULL)
+	{
+		cb(_core, service, user_data);
+		handle = galago_calls_request_dummy_handle();
+	}
+	else if (!galago_is_daemon() && galago_is_connected())
+	{
+		handle = galago_dbus_send_message_with_reply_async(
+			GALAGO_OBJECT(_core), "GetService", G_CALLBACK(cb),
+			user_data, free_func, g_cclosure_marshal_VOID__POINTER,
+			galago_value_new_object(GALAGO_TYPE_SERVICE, NULL),
+			galago_value_new(GALAGO_VALUE_TYPE_STRING, &id, NULL),
+			NULL);
+	}
+
+	galago_context_pop();
+
+	return handle;
+}
+
 /**
  * galago_get_services
  * @origin: The desired origin.
