@@ -20,11 +20,98 @@
  */
 #include <libgalago/galago-dbus.h>
 #include <libgalago/galago-assert.h>
+#include <libgalago/galago-context.h>
 #include <libgalago/galago-core.h>
 #include <libgalago/galago-status.h>
 #include <string.h>
 #include <stdio.h>
 
+typedef void (*PendingCallCb)(const GalagoObject *, GList *, gpointer);
+
+typedef struct
+{
+	const GalagoObject *object;
+	char *name;
+	GList *return_types;
+	PendingCallCb cb;
+	gpointer user_data;
+	GalagoCallHandle handle;
+
+} PendingCallData;
+
+G_LOCK_DEFINE_STATIC(_calls_lock);
+static guint _next_call_handle = 1;
+static GHashTable *_active_calls = NULL;
+
+static void
+pending_call_data_free(PendingCallData *data)
+{
+	if (data->return_types != NULL)
+	{
+		g_list_foreach(data->return_types, (GFunc)galago_value_destroy, NULL);
+		g_list_free(data->return_types);
+	}
+
+	g_free(data->name);
+	g_free(data);
+}
+
+/**
+ * galago_call_cancel:
+ * @handle: The call handle to cancel.
+ *
+ * Cancels an asynchronous call.
+ */
+void
+galago_call_cancel(GalagoCallHandle handle)
+{
+	DBusPendingCall *call;
+
+	G_LOCK(_calls_lock);
+
+	call = g_hash_table_lookup(_active_calls, GINT_TO_POINTER(handle));
+
+	if (call != NULL)
+	{
+		g_hash_table_remove(_active_calls, GINT_TO_POINTER(handle));
+		dbus_pending_call_cancel(call);
+	}
+
+	G_UNLOCK(_calls_lock);
+}
+
+GalagoCallHandle
+galago_calls_request_dummy_handle(void)
+{
+	GalagoCallHandle handle;
+
+	G_LOCK(_calls_lock);
+	handle = _next_call_handle++;
+	G_UNLOCK(_calls_lock);
+
+	return handle;
+}
+
+static GalagoCallHandle
+galago_calls_request_handle(DBusPendingCall *call)
+{
+	GalagoCallHandle handle;
+
+	g_return_val_if_fail(call != NULL, GALAGO_CALL_HANDLE_INVALID);
+
+	G_LOCK(_calls_lock);
+
+	if (_active_calls == NULL)
+		_active_calls = g_hash_table_new(g_direct_hash, g_direct_equal);
+
+	handle = _next_call_handle++;
+	g_hash_table_insert(_active_calls, GINT_TO_POINTER(handle), call);
+
+	G_UNLOCK(_calls_lock);
+
+	return handle;
+}
+
 /**
  * galago_dbus_message_iter_append_string_or_nil
  * @iter: The message iterator.
@@ -143,7 +230,6 @@ galago_dbus_message_iter_append_object_list(DBusMessageIter *iter, GType type,
  * @iter: The D-BUS message iterator.
  * @type: The GType.
  *
- *
  * Returns an object of the specified type from a D-BUS message.
  *
  * Returns: The object.
@@ -649,17 +735,99 @@ get_ret_val_from_iter(DBusMessageIter *iter, GalagoValue *value)
 	return retval;
 }
 
+static GList *
+message_reply_received_cb(DBusMessage *message, GList *return_types)
+{
+	GList *ret_list = NULL, *l;
+	DBusMessageIter iter;
+
+	dbus_message_iter_init(message, &iter);
+
+	for (l = return_types; l != NULL; l = l->next)
+	{
+		GalagoValue *value = (GalagoValue *)l->data;
+		ret_list = g_list_append(ret_list,
+								 get_ret_val_from_iter(&iter, value));
+		galago_value_destroy(value);
+
+		dbus_message_iter_next(&iter);
+	}
+
+	g_list_free(return_types);
+
+	return ret_list;
+}
+
+static void
+handle_pending_call_error(const GalagoObject *object, const char *name,
+						  DBusError *error)
+{
+	if (!dbus_error_has_name(error, GALAGO_DBUS_ERROR_OBJECT_NOT_FOUND))
+	{
+		g_warning("Error sending %s.%s: %s",
+				  g_type_name(G_OBJECT_TYPE(object)), name, error->message);
+	}
+}
+
+static void
+pending_call_done_cb(DBusPendingCall *call, gpointer user_data)
+{
+	PendingCallData *call_data = (PendingCallData *)user_data;
+	DBusMessage *reply = NULL;
+	GList *ret_list;
+
+	if (!dbus_pending_call_get_completed(call))
+		goto exit;
+
+	reply = dbus_pending_call_steal_reply(call);
+
+	if (reply == NULL)
+		goto exit;
+
+	if (dbus_message_get_type(reply) == DBUS_MESSAGE_TYPE_ERROR)
+	{
+		DBusError error;
+
+		dbus_error_init(&error);
+		dbus_set_error_from_message(&error, reply);
+		handle_pending_call_error(call_data->object, call_data->name, &error);
+		dbus_error_free(&error);
+
+		goto exit;
+	}
+
+	galago_context_push(galago_object_get_context(call_data->object));
+	ret_list = message_reply_received_cb(reply, call_data->return_types);
+	galago_context_pop();
+
+	call_data->cb(call_data->object, ret_list, call_data->user_data);
+
+	/* Hack to prevent this from being freed later. */
+	call_data->return_types = NULL;
+
+exit:
+	if (reply != NULL)
+		dbus_message_unref(reply);
+
+	G_LOCK(_calls_lock);
+	g_hash_table_remove(_active_calls, GINT_TO_POINTER(call_data->handle));
+	G_UNLOCK(_calls_lock);
+
+	dbus_pending_call_unref(call);
+}
+
 static GList *
 galago_dbus_send_message_with_reply_list_vargs(const GalagoObject *object,
 											   const char *name,
 											   GList *return_types,
-											   va_list args)
+											   va_list args,
+											   PendingCallCb cb,
+											   gpointer user_data,
+											   GalagoCallHandle *handle)
 {
-	DBusMessage *message;
-	DBusMessage *reply = NULL;
-	DBusMessageIter iter;
+	DBusMessage *message = NULL;
 	DBusError error;
-	GList *ret_list = NULL, *l;
+	GList *ret_list = NULL;
 
 	galago_goto_if_fail(object != NULL,           exit);
 	galago_goto_if_fail(name   != NULL,           exit);
@@ -677,45 +845,54 @@ galago_dbus_send_message_with_reply_list_vargs(const GalagoObject *object,
 
 	galago_goto_if_fail(message != NULL, exit);
 
-	reply = dbus_connection_send_with_reply_and_block(
-		galago_get_dbus_conn(), message, -1, &error);
-
-	dbus_message_unref(message);
-
-	if (dbus_error_is_set(&error))
+	if (cb == NULL)
 	{
-		if (!dbus_error_has_name(&error, GALAGO_DBUS_ERROR_OBJECT_NOT_FOUND))
+		/* Synchronous */
+		DBusMessage *reply;
+
+		reply = dbus_connection_send_with_reply_and_block(
+			galago_get_dbus_conn(), message, -1, &error);
+
+		if (dbus_error_is_set(&error))
 		{
-			g_warning("Error sending %s.%s: %s",
-					  g_type_name(G_OBJECT_TYPE(object)),
-					  name, error.message);
+			handle_pending_call_error(object, name, &error);
+			goto exit;
 		}
 
-		goto exit;
+		galago_context_push(galago_object_get_context(object));
+		ret_list = message_reply_received_cb(reply, return_types);
+		galago_context_pop();
 	}
-
-	dbus_message_iter_init(reply, &iter);
-
-	for (l = return_types; l != NULL; l = l->next)
+	else
 	{
-		ret_list = g_list_append(ret_list,
-			get_ret_val_from_iter(&iter, (GalagoValue *)l->data));
+		/* Asynchronous */
+		DBusPendingCall *call = NULL;
+		PendingCallData *call_data;
 
-		dbus_message_iter_next(&iter);
+		dbus_connection_send_with_reply(galago_get_dbus_conn(),
+										message, &call, -1);
+
+		call_data = g_new0(PendingCallData, 1);
+		call_data->object = object;
+		call_data->name = g_strdup(name);
+		call_data->return_types = return_types;
+		call_data->cb = cb;
+		call_data->user_data = user_data;
+		call_data->handle = galago_calls_request_handle(call);
+
+		if (handle != NULL)
+			*handle = call_data->handle;
+
+		dbus_pending_call_set_notify(call, pending_call_done_cb, call_data,
+									 (DBusFreeFunction)pending_call_data_free);
 	}
 
 exit:
+	if (message != NULL)
+		dbus_message_unref(message);
+
 	dbus_error_free(&error);
 
-	if (reply != NULL)
-		dbus_message_unref(reply);
-
-	if (return_types != NULL)
-	{
-		g_list_foreach(return_types, (GFunc)galago_value_destroy, NULL);
-		g_list_free(return_types);
-	}
-
 	return ret_list;
 }
 
@@ -744,7 +921,8 @@ galago_dbus_send_message_with_reply_list(const GalagoObject *object,
 
 	va_start(args, return_types);
 	list = galago_dbus_send_message_with_reply_list_vargs(object, name,
-														  return_types, args);
+														  return_types, args,
+														  NULL, NULL, NULL);
 	va_end(args);
 
 	return list;
@@ -775,7 +953,8 @@ galago_dbus_send_message_with_reply(const GalagoObject *object,
 
 	va_start(args, return_type);
 	list = galago_dbus_send_message_with_reply_list_vargs(
-		object, name, g_list_append(NULL, return_type), args);
+		object, name, g_list_append(NULL, return_type), args,
+		NULL, NULL, NULL);
 	va_end(args);
 
 	if (list != NULL)
@@ -788,6 +967,60 @@ galago_dbus_send_message_with_reply(const GalagoObject *object,
 	return retval;
 }
 
+static void
+galago_dbus_send_message_with_reply_async_cb(const GalagoObject *object,
+											 GList *list,
+											 gpointer user_data)
+{
+	GClosure *closure = (GClosure *)user_data;
+	GValue params[2] = {{0, }, {0, }};
+
+	/* For now.. */
+	g_assert(g_list_length(list) == 1);
+	g_assert(list != NULL);
+	g_assert(GALAGO_IS_OBJECT(list->data));
+
+	g_value_init(&params[0], G_TYPE_FROM_INSTANCE(object));
+	g_value_set_instance(&params[0], (gpointer)object);
+
+	g_value_init(&params[1], G_TYPE_FROM_INSTANCE(list->data));
+	g_value_set_instance(&params[1], list->data);
+
+	g_closure_invoke(closure, NULL, 2, params, NULL);
+	g_closure_unref(closure);
+
+	g_value_unset(&params[0]);
+	g_value_unset(&params[1]);
+}
+
+GalagoCallHandle
+galago_dbus_send_message_with_reply_async(const GalagoObject *object,
+										  const char *name,
+										  GCallback cb, gpointer user_data,
+										  GFreeFunc destroy_data,
+										  GClosureMarshal marshal,
+										  GalagoValue *return_type, ...)
+{
+	va_list args;
+	GalagoCallHandle handle;
+	GClosure *closure;
+
+	closure = g_cclosure_new(cb, user_data, (GClosureNotify)destroy_data);
+	g_closure_ref(closure);
+	g_closure_sink(closure);
+	g_closure_set_marshal(closure, marshal);
+	g_object_watch_closure(G_OBJECT(object), closure);
+
+	va_start(args, return_type);
+	galago_dbus_send_message_with_reply_list_vargs(
+		object, name, g_list_append(NULL, return_type), args,
+		galago_dbus_send_message_with_reply_async_cb, closure, &handle);
+	va_end(args);
+
+	return handle;
+}
+
+
 /**
  * galago_dbus_object_push_full
  * @object: The object to push.
