@@ -706,13 +706,11 @@ galago_person_get_accounts(const GalagoPerson *person, gboolean query)
 	if (query && GALAGO_OBJECT_IS_REMOTE(person) && !galago_is_daemon() &&
 		galago_is_connected())
 	{
-		galago_context_push(galago_object_get_context(GALAGO_OBJECT(person)));
 		g_list_free(galago_dbus_send_message_with_reply(GALAGO_OBJECT(person),
 			"GetAccounts",
 			galago_value_new_list(GALAGO_VALUE_TYPE_OBJECT, NULL,
 								  (void *)GALAGO_TYPE_ACCOUNT),
 			NULL));
-		galago_context_pop();
 	}
 
 	return person->priv->accounts;
@@ -732,7 +730,7 @@ _galago_person_default_calc_priority_account(const GalagoPerson *person)
 		GalagoAccount *account = (GalagoAccount *)l->data;
 		GalagoPresence *presence = galago_account_get_presence(account, TRUE);
 
-		if (galago_presence_compare(priority_presence, presence) > 0)
+		if (galago_presence_compare(priority_presence, presence) < 0)
 			priority_presence = presence;
 	}
 
@@ -873,16 +871,12 @@ galago_person_get_account(const GalagoPerson *person,
 	if (account == NULL && query && GALAGO_OBJECT_IS_REMOTE(person) &&
 		!galago_is_daemon() && galago_is_connected())
 	{
-		galago_context_push(galago_object_get_context(GALAGO_OBJECT(person)));
-
 		account = galago_dbus_send_message_with_reply(
 			GALAGO_OBJECT(person), "GetAccount",
 			galago_value_new_object(GALAGO_TYPE_ACCOUNT, NULL),
 			galago_value_new_object(GALAGO_TYPE_SERVICE, G_OBJECT(service)),
 			galago_value_new(GALAGO_VALUE_TYPE_STRING, &username, NULL),
 			NULL);
-
-		galago_context_pop();
 	}
 
 	return account;
