@@ -406,7 +406,7 @@ galago_presence_set_idle(GalagoPresence *presence, gboolean idle,
 
 	g_return_if_fail(presence != NULL);
 	g_return_if_fail(GALAGO_IS_PRESENCE(presence));
-	g_assert(!idle || idle_start_time > 0);
+	g_return_if_fail(!idle || idle_start_time >= 0);
 
 	if (presence->priv->idle == idle &&
 		presence->priv->idle_start_time == idle_start_time)
@@ -928,9 +928,9 @@ galago_presence_compare(const GalagoPresence *presence1,
 	if (presence1 == presence2)
 		return 0;
 	else if (presence1 == NULL)
-		return 1;
-	else if (presence2 == NULL)
 		return -1;
+	else if (presence2 == NULL)
+		return 1;
 
 	/* Compute the score of the first set of statuses. */
 	for (l = galago_presence_get_statuses(presence1); l != NULL; l = l->next)
@@ -958,9 +958,9 @@ galago_presence_compare(const GalagoPresence *presence1,
 		if (!idle1 && !idle2)
 			return 0;
 		else if (idle1 && !idle2)
-			return 1;
-		else if (!idle1 && idle2)
 			return -1;
+		else if (!idle1 && idle2)
+			return 1;
 		else
 		{
 			time_t idle_time_1;
