@@ -38,6 +38,7 @@ struct _GalagoServicePrivate
 
 	GHashTable *accounts_table;
 	GList *accounts;
+	GList *registered_statuses;
 };
 
 enum
@@ -140,21 +141,24 @@ galago_service_class_init(GalagoServiceClass *klass)
 						   "The service flags",
 						   GALAGO_TYPE_SERVICE_FLAGS, 0,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-						   G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+						   G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
+						   G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property(gobject_class, PROP_ID,
 		g_param_spec_string("id", "ID",
 						   "The service's unique ID",
 						   NULL,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-						   G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+						   G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
+						   G_PARAM_STATIC_BLURB));
 
 	g_object_class_install_property(gobject_class, PROP_NAME,
 		g_param_spec_string("name", "Name",
 						   "The service's name",
 						   NULL,
 						   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
-						   G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+						   G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK |
+						   G_PARAM_STATIC_BLURB));
 }
 
 static void
@@ -566,6 +570,7 @@ galago_service_get_account(const GalagoService *service, const char *username,
 
 	g_return_val_if_fail(service  != NULL,           NULL);
 	g_return_val_if_fail(username != NULL,           NULL);
+	g_return_val_if_fail(*username != '\0',          NULL);
 	g_return_val_if_fail(GALAGO_IS_SERVICE(service), NULL);
 
 	norm_username = galago_service_normalize(service, username);
@@ -576,20 +581,90 @@ galago_service_get_account(const GalagoService *service, const char *username,
 	if (account == NULL && query && GALAGO_OBJECT_IS_REMOTE(service) &&
 		!galago_is_daemon() && galago_is_connected())
 	{
-		galago_context_push(galago_object_get_context(GALAGO_OBJECT(service)));
-
 		account = galago_dbus_send_message_with_reply(
 			GALAGO_OBJECT(service), "GetAccount",
 			galago_value_new_object(GALAGO_TYPE_ACCOUNT, NULL),
 			galago_value_new(GALAGO_VALUE_TYPE_STRING, &username, NULL),
 			NULL);
-
-		galago_context_pop();
 	}
 
 	return account;
 }
 
+/**
+ * galago_service_get_account_async
+ * @service:   The service.
+ * @username:  The account's username.
+ * @cb:        The callback function that will be passed the account.
+ * @user_data: Custom data to pass to the callback function.
+ * @free_func: Optional function to free @user_data when the request completes.
+ *
+ * Asynchronously retrieves the account with the specified username from
+ * a service.
+ *
+ * For example:
+ * <informalexample>
+ * <programlisting>
+ *  static void
+ *  account_received_cb(GalagoService *service,
+ *                      GalagoAccount *account,
+ *                      gpointer user_data)
+ *  {
+ *  }
+ *  
+ *  static void
+ *  get_bob_account(GalagoService *service)
+ *  {
+ *      GalagoCallHandle handle;
+ *  
+ *      handle = galago_service_get_account_async(service, "Bob",
+ *                                                account_received_cb,
+ *                                                NULL, NULL);
+ *  }
+ * </programlisting>
+ * </informalexample>
+ *
+ * Returns: A valid #GalagoCallHandle on success. On error, this will return
+ *          %GALAGO_CALL_HANDLE_INVALID.
+ *
+ * Since: 0.5.2
+ */
+GalagoCallHandle
+galago_service_get_account_async(const GalagoService *service,
+								 const char *username,
+								 GalagoServiceAccountCb cb,
+								 gpointer user_data,
+								 GFreeFunc free_func)
+{
+	GalagoAccount *account;
+	GalagoCallHandle handle = GALAGO_CALL_HANDLE_INVALID;
+
+	g_return_val_if_fail(service  != NULL,                      handle);
+	g_return_val_if_fail(GALAGO_IS_SERVICE(service),            handle);
+	g_return_val_if_fail(username != NULL && *username != '\0', handle);
+	g_return_val_if_fail(cb != NULL,                            handle);
+
+	account = galago_service_get_account(service, username, FALSE);
+
+	if (account != NULL)
+	{
+		cb((GalagoService *)service, account, user_data);
+		handle = galago_calls_request_dummy_handle();
+	}
+	else if (GALAGO_OBJECT_IS_REMOTE(service) && !galago_is_daemon() &&
+			 galago_is_connected())
+	{
+		handle = galago_dbus_send_message_with_reply_async(
+			GALAGO_OBJECT(service), "GetAccount", G_CALLBACK(cb),
+			user_data, free_func, g_cclosure_marshal_VOID__POINTER,
+			galago_value_new_object(GALAGO_TYPE_ACCOUNT, NULL),
+			galago_value_new(GALAGO_VALUE_TYPE_STRING, &username, NULL),
+			NULL);
+	}
+
+	return handle;
+}
+
 /**
  * galago_service_get_accounts
  * @service: The service.
@@ -616,16 +691,12 @@ galago_service_get_accounts(const GalagoService *service, gboolean query)
 	{
 		GList *temp;
 
-		galago_context_push(galago_object_get_context(GALAGO_OBJECT(service)));
-
 		temp = galago_dbus_send_message_with_reply(
 			GALAGO_OBJECT(service), "GetAccounts",
 			galago_value_new_list(GALAGO_VALUE_TYPE_OBJECT, NULL,
 								  (void *)GALAGO_TYPE_ACCOUNT),
 			NULL);
 		g_list_free(temp);
-
-		galago_context_pop();
 	}
 
 	return service->priv->accounts;
