@@ -353,6 +353,134 @@ START_TEST(test_service_normalize)
 END_TEST
 
 
+/**************************************************************************
+ * Test synchronous calls
+ **************************************************************************/
+START_TEST(test_service_get_account)
+{
+	GalagoService *service;
+	GalagoAccount *account;
+
+	galago_init("check-libgalago", GALAGO_INIT_CLIENT);
+
+	service = galago_get_service("aim", GALAGO_REMOTE, TRUE);
+	fail_unless(service != NULL,
+				"Couldn't fetch aim service. Is presence-feed running?");
+
+	account = galago_service_get_account(service, "GalagoAIMUser", TRUE);
+	fail_unless(service != NULL,
+				"Couldn't get account GalagoAIMUser. "
+				"Is presence-feed running?");
+}
+END_TEST
+
+
+/**************************************************************************
+ * Test asynchronous calls
+ **************************************************************************/
+static void
+service_received_cb(GalagoCore *core, GalagoService *service,
+					gpointer user_data)
+{
+	GMainLoop *loop = (GMainLoop *)user_data;
+
+	fail_unless(service != NULL && GALAGO_IS_SERVICE(service) &&
+				!strcmp(galago_service_get_id(service), "aim"),
+				"Retrieved incorrect service parameter.");
+
+	g_main_loop_quit(loop);
+}
+
+START_TEST(test_core_get_service_async)
+{
+	GMainLoop *loop;
+
+	loop = g_main_loop_new(NULL, FALSE);
+	galago_init("check-libgalago", GALAGO_INIT_CLIENT);
+	galago_get_service_async("aim", service_received_cb, loop, NULL);
+	g_main_loop_run(loop);
+}
+END_TEST
+
+
+static void
+account_received_cb(GalagoService *service, GalagoAccount *account,
+					gpointer user_data)
+{
+	GMainLoop *loop = (GMainLoop *)user_data;
+
+	fail_unless(service != NULL && GALAGO_IS_SERVICE(service) &&
+				!strcmp(galago_service_get_id(service), "aim"),
+				"Retrieved incorrect service parameter.");
+
+	fail_unless(account != NULL && GALAGO_IS_ACCOUNT(account) &&
+				!strcmp(galago_account_get_username(account), "GalagoAIMUser"),
+				"Retrieved incorrect account parameter.");
+
+	g_main_loop_quit(loop);
+}
+
+START_TEST(test_service_get_account_async)
+{
+	GalagoService *service;
+	GMainLoop *loop;
+
+	loop = g_main_loop_new(NULL, FALSE);
+
+	galago_init("check-libgalago", GALAGO_INIT_CLIENT);
+
+	service = galago_get_service("aim", GALAGO_REMOTE, TRUE);
+	fail_unless(service != NULL,
+				"Couldn't fetch aim service. Is presence-feed running?");
+
+	galago_service_get_account_async(service, "GalagoAIMUser",
+									 account_received_cb, loop, NULL);
+
+	g_main_loop_run(loop);
+}
+END_TEST
+
+static void
+presence_received_cb(GalagoAccount *account, GalagoPresence *presence,
+					 gpointer user_data)
+{
+	GMainLoop *loop = (GMainLoop *)user_data;
+
+	fail_unless(account != NULL && GALAGO_IS_ACCOUNT(account) &&
+				!strcmp(galago_account_get_username(account), "GalagoAIMUser"),
+				"Retrieved incorrect account parameter.");
+
+	fail_unless(presence != NULL && GALAGO_IS_PRESENCE(presence),
+				"Retrieved incorrect presence parameter.");
+
+	g_main_loop_quit(loop);
+}
+
+START_TEST(test_account_get_presence_async)
+{
+	GalagoService *service;
+	GalagoAccount *account;
+	GMainLoop *loop;
+
+	loop = g_main_loop_new(NULL, FALSE);
+
+	galago_init("check-libgalago", GALAGO_INIT_CLIENT);
+
+	service = galago_get_service("aim", GALAGO_REMOTE, TRUE);
+	fail_unless(service != NULL,
+				"Couldn't fetch aim service. Is presence-feed running?");
+
+	account = galago_service_get_account(service, "GalagoAIMUser", TRUE);
+	fail_unless(service != NULL,
+				"Couldn't fetch GalagoAIMUser. Is presence-feed running?");
+
+	galago_account_get_presence_async(account, presence_received_cb, loop,
+									  NULL);
+
+	g_main_loop_run(loop);
+}
+END_TEST
+
 /**************************************************************************
  * Core tests
  **************************************************************************/
@@ -402,6 +530,14 @@ make_libgalago_suite(void)
 	/* Test the Service object */
 	ADD_TCASE("service_normalize", test_service_normalize);
 
+	/* Test synchronous queries */
+	ADD_TCASE("service_get_account", test_service_get_account);
+
+	/* Test asynchronous queries */
+	ADD_TCASE("core_get_service_async", test_core_get_service_async);
+	ADD_TCASE("service_get_account_async", test_service_get_account_async);
+	ADD_TCASE("account_get_presence_async", test_account_get_presence_async);
+
 	return s;
 }
 
